unit fuAQBuilder;

interface

uses Windows, Classes, Graphics, Forms, Controls, StdCtrls, SynMemo,
  SynEdit, ExtCtrls, ToolWin, ComCtrls, acQBBase, DB, ADODB, SysUtils,
  DBCtrls, Grids, DBGrids, ImgList, Dialogs,
  acSQL92SynProvider, acSQLBuilderPlainText, acQBADOMetaProvider,
  {$IFNDEF FREE} acMYSQLSynProvider, acMSSQLSynProvider, acMSSQLAST, {$ENDIF}
  SynEditHighlighter, SynHighlighterSQL;

type
  TfmAQBuilder = class(TForm)
    acADOMetadataProvider: TacADOMetadataProvider;
    acSQLBuilderPlainText: TacSQLBuilderPlainText;
    ADOConnection: TADOConnection;
    panelBuilder: TPanel;
    splVertical: TSplitter;
    panelBuilderLeft: TPanel;
    splHorizontal: TSplitter;
    editSQL: TSynMemo;
    QueryBuilder: TacQueryBuilder;
    listTables: TListView;
    panelGrid: TPanel;
    panelGridTop: TPanel;
    DataSource: TDataSource;
    Query: TADOQuery;
    Grid: TDBGrid;
    DBNavigator: TDBNavigator;
    panelBuilderTop: TPanel;
    cbSyntaxProvider: TComboBox;
    lbSyntaxProvider: TLabel;
    bvTop: TBevel;
    Bevel2: TBevel;
    imglistTable: TImageList;
    SynSQLSyntaxHighlighter: TSynSQLSyn;
    pnlServerVersion: TPanel;
    cbMSVersion: TComboBox;
    edMyVersion: TEdit;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cbSyntaxProviderChange(Sender: TObject);
    procedure QueryBuilderDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure QueryBuilderDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure acSQLBuilderPlainTextSQLUpdated(Sender: TObject);
    procedure editSQLExit(Sender: TObject);
    procedure editSQLChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure listTablesDblClick(Sender: TObject);
    procedure QueryBuilderSQLError(Sender: TObject; ASQL, AErrors: String);
    procedure cbMSVersionChange(Sender: TObject);
    procedure edMyVersionChange(Sender: TObject);
  private
    { Private declarations }
    fSQLChanged: Boolean;
    fEditorChanged: Boolean;
    fRun: Boolean;
    fSyntaxProvider: TacBaseSyntaxProvider;
    procedure SetRun(const Value: Boolean);
    procedure SetEditorChanged(const Value: Boolean);
  public
    { Public declarations }
    property IsRun: Boolean read fRun write SetRun;
    property EditorChanged: Boolean read fEditorChanged write SetEditorChanged;
  end;

implementation

{$R *.dfm}

uses fuAQBDemoMain;

procedure TfmAQBuilder.FormCreate(Sender: TObject);
begin
	IsRun := False;
	fSQLChanged := False;
	fEditorChanged := False;

	with cbSyntaxProvider.Items do begin
		Add('ANSI 92');
		{$IFNDEF FREE}
		Add('Microsoft SQL Server');
		Add('MySQL');
		{$ENDIF}
	end;
	cbSyntaxProvider.ItemIndex := 0;
	cbSyntaxProviderChange(nil);
end;

procedure TfmAQBuilder.FormActivate(Sender: TObject);
begin with MainForm do begin
	CurrentQueryBuilder := Self;

	acQueryBuild.Enabled := fRun;
	acQueryRun.Enabled := not fRun;
	acFileSave.Enabled := fEditorChanged;
	acFileSaveAs.Enabled := True;

end end;

procedure TfmAQBuilder.FormClose(Sender: TObject; var Action: TCloseAction);
begin
	Action := caFree;
end;

procedure TfmAQBuilder.cbSyntaxProviderChange(Sender: TObject);
begin
	if Assigned(fSyntaxProvider)
		then FreeAndNil(fSyntaxProvider);

	pnlServerVersion.Visible := cbSyntaxProvider.ItemIndex <> 0;
	case cbSyntaxProvider.ItemIndex of
		0: fSyntaxProvider := TacSQL92SyntaxProvider.Create(Self);
		{$IFNDEF FREE}
		1: begin
				fSyntaxProvider := TacMSSQLSyntaxProvider.Create(Self);
				cbSyntaxProvider.BringToFront;
				case TacMSSQLSyntaxProvider(fSyntaxProvider).ServerVersion of
					svMSSQL7:    cbSyntaxProvider.ItemIndex := 0;
					svMSSQL2000: cbSyntaxProvider.ItemIndex := 1;
					svMSSQL2005: cbSyntaxProvider.ItemIndex := 2;
				end;
			end;
		2: begin
				fSyntaxProvider := TacMYSQLSyntaxProvider.Create(Self);
				edMyVersion.BringToFront;
				edMyVersion.Text :=
					TacMYSQLSyntaxProvider(fSyntaxProvider).ServerVersion;
			end;
		{$ENDIF}
	end;

	QueryBuilder.SyntaxProvider := fSyntaxProvider;
end;

procedure TfmAQBuilder.QueryBuilderDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
	Accept := (Source = listTables); //and (Sender = QueryBuilder)};
end;

procedure TfmAQBuilder.QueryBuilderDragDrop(Sender, Source: TObject; X,
	Y: Integer);
var
	ds: TacDataSource;
	usq: TacUnionSubQuery;
	objName: string;
begin
	usq := Sender as TacUnionSubQuery;
	usq.BeginUpdate;
	try
		objName := listTables.Items[listTables.ItemIndex].Caption;
		ds := usq.AddObjectAt(
				QueryBuilder.SyntaxProvider.QuoteBegin +
				objName + QueryBuilder.SyntaxProvider.QuoteEnd,
				Point(x,y));
      try
			if ds is TacDataSourceObject
				then usq.AddFKLinksForObject(TacDataSourceObject(ds));
		except
		end;
	finally
		usq.EndUpdate;
	end;
end;

procedure TfmAQBuilder.listTablesDblClick(Sender: TObject);
var
	ds: TacDataSource;
	usq: TacUnionSubQuery;
	objName: string;
begin
	usq := QueryBuilder.ActiveSubQuery.ActiveUnionSubquery;
	usq.BeginUpdate;
	try
		objName := listTables.Items[listTables.ItemIndex].Caption;
		ds := usq.AddObject(
				QueryBuilder.SyntaxProvider.QuoteBegin +
            objName + QueryBuilder.SyntaxProvider.QuoteEnd);
		try
			if ds is TacDataSourceObject
				then usq.AddFKLinksForObject(TacDataSourceObject(ds));
		except
		end;
	finally
		usq.EndUpdate;
	end;
end;

procedure TfmAQBuilder.acSQLBuilderPlainTextSQLUpdated(Sender: TObject);
begin
	editSQL.Text := acSQLBuilderPlainText.SQL;
	EditorChanged := True;
end;

procedure TfmAQBuilder.editSQLExit(Sender: TObject);
begin
	if fSQLChanged
	then begin
		QueryBuilder.SQL := editSQL.Text;
		fSQLChanged := False;
	end;
end;

procedure TfmAQBuilder.editSQLChange(Sender: TObject);
begin
	fSQLChanged := True;
	EditorChanged := True;
end;

procedure TfmAQBuilder.SetRun(const Value: Boolean);
begin
	fRun := Value;
	if Value
	then begin
		Query.SQL.Assign(editSQL.Lines);
		Query.Prepared := True;
		if Query.Prepared
		then begin
			Query.Open;
			panelGrid.BringToFront;
		end;
	end
	else begin
		if Query.Active then Query.Close;
		panelBuilder.BringToFront;
	end;
	FormActivate(nil);
end;

procedure TfmAQBuilder.SetEditorChanged(const Value: Boolean);
begin
	fEditorChanged := Value;
	FormActivate(nil);
end;

procedure TfmAQBuilder.QueryBuilderSQLError(Sender: TObject; ASQL,
  AErrors: String);
begin
	ShowMessage(ASQL + #13#10 + AErrors);
end;

procedure TfmAQBuilder.cbMSVersionChange(Sender: TObject);
begin
	if fSyntaxProvider is TacMSSQLSyntaxProvider
	then with fSyntaxProvider as TacMSSQLSyntaxProvider do begin

		case cbMSVersion.ItemIndex of
			0: ServerVersion := svMSSQL7;
			1: ServerVersion := svMSSQL2000;
			2: ServerVersion := svMSSQL2005;
		end;
	end;
end;

procedure TfmAQBuilder.edMyVersionChange(Sender: TObject);
begin
	if fSyntaxProvider is TacMySQLSyntaxProvider
	then with fSyntaxProvider as TacMySQLSyntaxProvider do begin

		ServerVersion := edMyVersion.Text;
		edMyVersion.Text := ServerVersion;
	end;
end;

end.
